
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "base/polygraph.h"

#include <stdlib.h>
#include "xstd/h/os_std.h"
#include "xstd/h/process.h" /* for _getpid() on W2K */
#include "xstd/h/iostream.h"
#include "xstd/h/strstream.h"
#include "xstd/h/iomanip.h"

#include "base/ILog.h"
#include "base/OLog.h"
#include "base/UniqId.h"
#include "xstd/gadgets.h"

static UniqId TheSessionId;
static int TheSpaceId = 0;


void UniqId::Space(int id) {
	// we must be called before any ids are generated (weak check)
	Assert(TheSessionId.theCnt <= 0); 
	Assert(id >= 0); 
	TheSpaceId = id;
}

UniqId::UniqId(): theSecs(0), theMix(0), theCnt(0) {
}

UniqId::UniqId(int aSecs, int aMix, int aCnt): theSecs(aSecs), theMix(aMix), theCnt(aCnt) {
	Assert(aCnt <= 0 || (aSecs > 0 && aMix > 0));
}

UniqId &UniqId::create() {
	if (TheSessionId.theCnt <= 0)
		RefreshSessionId();
	*this = TheSessionId;
	TheSessionId.theCnt += 2;
	return *this;
}

int UniqId::hash() const {
	return abs(theMix + (theCnt ^ theSecs));
}

ostream &UniqId::print(ostream &os) const {
	const ios_fmtflags flags = os.flags();
	const char fill = os.fill('0');
	os << hex
		<< setw(8) << theSecs << '.' 
		<< setw(8) << theMix << ':' 
		<< setw(8) << theCnt;
	os.fill(fill);
	os.flags(flags);
	return os;
}

OLog &UniqId::store(OLog &ol) const {
	return ol << theSecs << theMix << theCnt;
}

ILog &UniqId::load(ILog &il) {
	return il >> theSecs >> theMix >> theCnt;
}

String UniqId::str() const {
	char buf[64];
	ostrstream os(buf, sizeof(buf));
	print(os) << ends;
	return buf;
}

UniqId UniqId::FromStr(const String &s) {
	UniqId i;
	memcpy(&i, s.str(), Min(s.len(), (int)sizeof(i)));
	// make theCnt even and positive
	i.theCnt /= 2;
	i.theCnt *= 2;
	i.theCnt += 2;
	// call constructor to check for correctness
	return UniqId(i.theSecs, i.theMix, i.theCnt);
}

// it would be better to use hostid in a mix, but gethostid() is not portable
// so we use tm.usec instead
void UniqId::RefreshSessionId() {
	if (!TheSpaceId) {
		// use random factors to reset "session id"
		static unsigned pid = (unsigned) getpid();
		Time tm(Time::Now());

		TheSessionId = UniqId(
			0x7fffffff & (int)(tm.sec() - 912345678),
			0x7fffffff & (int)((tm.usec() << 16) | (pid & 0xffff)),
			2);
	} else {
		// stay within the specified space
		TheSessionId = UniqId(
			TheSpaceId,
			TheSessionId.theMix+1,
			2);
	}
}
