
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "pgl/pgl.h"

#include "xstd/String.h"
#include "pgl/PglRec.h"
#include "pgl/PglArraySym.h"
#include "pgl/PglNetAddrSym.h"
#include "pgl/HostsBasedSym.h"


HostsBasedSym::HostsBasedSym(const String &aType): RecSym(aType, new PglRec) {
	// originally, "addresses" were "hosts", and we did not want to
	// change class name when the field name has changed 
	// to avoid renaming a bunhc of files
	theRec->bAdd("addr[]", "addresses", 0);
}

HostsBasedSym::HostsBasedSym(const String &aType, PglRec *aRec): RecSym(aType, aRec) {
}

bool HostsBasedSym::isA(const String &type) const {
	return RecSym::isA(type) || type == "HostsBased";
}

SynSym *HostsBasedSym::dupe(const String &type) const {
	if (isA(type))
		return new HostsBasedSym(this->type(), theRec->clone());
	return RecSym::dupe(type);
}

int HostsBasedSym::hostCount() const {
	if (ArraySym *a = getArraySym("addresses"))
		return a->count();
	else
		return 0;
}

NetAddrSym *HostsBasedSym::host(int idx) const {
	if (ArraySym *a = getArraySym("addresses")) {
		Assert(idx < a->count());
		return &(NetAddrSym&)(*a)[idx]->cast("addr");
	}
	return 0;
}

bool HostsBasedSym::addresses(ArraySym &addrs) const {
	if (ArraySym *a = getArraySym("addresses")) {
		addrs.append(*a);
		return true;
	}
	return false;
}
