
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "base/polygraph.h"

#include "xml/XmlNode.h"
#include "xml/XmlAttr.h"
#include "xml/XmlAttrs.h"
#include "xml/XmlNodes.h"


XmlNodes::~XmlNodes() {
	while (count())
		delete pop();
}

int XmlNodes::selByAttrName(const String &name, XmlNodes &res) {
	for (int i = 0; i < count(); ++i) {
		XmlNode *node = item(i);
		if (node->attrs() && node->attrs()->has(name))
			res.append(node);
	}
	return res.count();
}

int XmlNodes::selByAttrVal(const String &name, const String &value, XmlNodes &res) {
	for (int i = 0; i < count(); ++i) {
		XmlNode *node = item(i);
		if (node->attrs() && node->attrs()->has(name, value))
			res.append(node);
	}
	return res.count();
}

XmlNodes &XmlNodes::operator <<(const XmlAttr &a) {
	for (int i = 0; i < count(); ++i)
		item(i)->addAttr(a.clone());
	return *this;
}

XmlNodes &XmlNodes::operator <<(const XmlNode &n) {
	append(n.clone());
	return *this;
}

XmlNodes &XmlNodes::operator <<(const XmlNodes &ns) {
	stretch(count() + ns.count());
	for (int i = 0; i < ns.count(); ++i)
		append(ns[i]->clone());
	return *this;
}

ostream &XmlNodes::print(ostream &os, const String &pfx) const {
	for (int i = 0; i < count(); ++i)
		item(i)->print(os, pfx);
	return os;
}
