
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "base/polygraph.h"

#include "xml/XmlAttr.h"
#include "xml/XmlTag.h"

XmlTag::XmlTag(const String &aName): theName(aName) {
}

XmlTag::~XmlTag() {
	while (theAttrs.count()) {
		XmlAttr *attr = theAttrs.pop();
		attr->node(0);
		delete attr;
	}

	while (theKids.count()) {
		XmlNode *child = theKids.pop();
		child->parent(0);
		delete child;
	}
}

XmlNode *XmlTag::clone() const {
	XmlTag *c = new XmlTag(theName);

	c->theAttrs.stretch(theAttrs.count());
	for (int a = 0; a < theAttrs.count(); ++a)
		c->theAttrs.append(theAttrs[a]->clone());

	c->theKids.stretch(theKids.count());
	for (int k = 0; k < theKids.count(); ++k)
		c->theKids.append(theKids[k]->clone());

	return c;
}

XmlAttr *XmlTag::append(const XmlAttr &a) {
	theAttrs << a;
	return theAttrs.last();
}

XmlNode *XmlTag::append(const XmlNode &n) {
	theKids << n;
	return theKids.last();
}

XmlTag &XmlTag::operator <<(const XmlAttr &a) {
	theAttrs << a;
	return *this;
}

XmlTag &XmlTag::operator <<(const XmlNode &n) {
	append(n);
	return *this;
}

XmlTag &XmlTag::operator <<(const XmlNodes &ns) {
	theKids << ns;
	return *this;
}

ostream &XmlTag::print(ostream &os, const String &pfx) const {
	os << pfx << '<' << theName;

	for (int a = 0; a < theAttrs.count(); ++a)
		theAttrs[a]->print(os, " ");

	// note: we add new lines and whitespace that may not have
	// been in the original document; should we do that?
	if (theKids.count()) {
		os << '>' << endl;
		for (int i = 0; i < theKids.count(); ++i)
			theKids[i]->print(os, pfx + "\t");
		if (!theKids.last()->attrs())
			os << endl;
		os << pfx << "</" << theName << '>' << endl;
	} else {
		os << "/>" << endl;
	}

	return os;
}
